@@ -363,7 +363,7 @@ class DefaultStorage(IDefaultStorable):
         return len(self._get_defaults) > 0
 
     def get(self) -> Any:
-        to_return = next((get_default() for condition, get_default in reversed(self._get_defaults.items()) if condition()), None)
+        to_return = next((get_default() for condition, get_default in reversed(self._get_defaults.items()) if condition()))  # TODO: add test for getting exception for empty or no condition met and check functions that use this function
         return to_return
 
 ###########################
@@ -430,7 +430,7 @@ class ImplicitlyActivableMixin(IActivable):
 
 class ConditionallyActiveMixin(IActivable, IHelp, ABC):
 
-    def __init__(self, active_condition: compositeActive = None, inactive_condition: compositeActive = None, default_state: bool = None, **kwargs):
+    def __init__(self, active_condition: compositeActive = None, inactive_condition: compositeActive = None, default_state: bool = False, **kwargs):
         super().__init__(**kwargs)
         self._active_conditions = SmartList(self._map_to_single(active_condition)) if active_condition else SmartList()
         self._inactive_conditions = SmartList(self._map_to_single(inactive_condition)) if inactive_condition else SmartList()
@@ -477,11 +477,11 @@ class ConditionallyActiveMixin(IActivable, IHelp, ABC):
     def set_inactive_on_flags(self, *flags: Flag, func=any):
         self.set_inactive_on_conditions(lambda: func([flag.is_active() for flag in flags]))
 
-    def set_active_on_flags_in_collection(self, collection: CliCollection, *flags: Flag, but_not: list[Flag] | Flag = None):
+    def set_active_on_flags_in_collection(self, collection: CliCollection, *flags: Flag, but_not: list[Flag] | Flag = None, func=all, but_not_func=any):
         but_not = but_not or []
         but_not = [but_not] if isinstance(but_not, Flag) else but_not
-        self.set_active_on_conditions(lambda: all((flag in collection for flag in flags)))
-        self.set_inactive_on_flags_in_collection(collection, *but_not, func=any)
+        self.set_active_on_conditions(lambda: func((flag in collection for flag in flags)))
+        self.set_inactive_on_flags_in_collection(collection, *but_not, func=but_not_func)
 
     def set_inactive_on_flags_in_collection(self, collection: CliCollection, *flags: Flag, func=all):
         self.set_inactive_on_conditions(lambda: func((flag in collection for flag in flags)))
@@ -615,13 +615,20 @@ class FlagManagerMixin:
     def get_active_flags(self) -> Iterable[Flag]:
         return filter(Flag.is_active, self._flags)
 
-    def add_flag(self, main: str | Flag, *alternative_names: str, storage: CliCollection = None, storage_limit: int | None = -1, storage_lower_limit=-1, default: default_type = None, flag_limit=-1, flag_lower_limit=-1) -> Flag:
+    def add_flag(self, main: str | Flag, *alternative_names: str, storage: CliCollection = None, storage_limit: int | None = -1, storage_lower_limit=-1, default: default_type = None, flag_limit=-1, flag_lower_limit=-1, multi=False) -> Flag:
         name = get_name(main)
         if self.has_flag(name):
             raise ValueAlreadyExistsError(Flag, name)
         flag = main if isinstance(main, Flag) else Flag(name, *alternative_names, storage=storage, storage_limit=storage_limit,
-                                                        storage_lower_limit=storage_lower_limit, flag_limit=flag_limit, default=default,
+                                                          storage_lower_limit=storage_lower_limit, flag_limit=flag_limit, default=default,
                                                         flag_lower_limit=flag_lower_limit)
+
+        if multi: # TODO: add tests for new args
+            if flag_lower_limit:
+                flag.set_to_multi(flag_lower_limit)
+            else:
+                flag.set_to_multi_at_least_one()
+
         self._flags.append(flag)
         return flag
 
@@ -699,7 +706,8 @@ class ParameterManagerMixin(IResetable):
             if not self.has_param(param):
                 self.add_param(param)
 
-    def add_param(self, to_add: str | Parameter | CliCollection, storage: CliCollection = None) -> Parameter:
+    # TODO: add tests for new args
+    def add_param(self, to_add: str | Parameter | CliCollection, storage: CliCollection = None, multi=False, lower_limit=None) -> Parameter:
         if storage is not None and isinstance(to_add, CliCollection):
             raise ValueError
 
@@ -707,13 +715,22 @@ class ParameterManagerMixin(IResetable):
             storage = to_add
             to_add = to_add.name
 
-        name, param = get_name_and_object_for_namable(to_add, Parameter)
+        if isinstance(to_add, str):
+            to_add = Parameter(to_add, storage=storage)
+
+        name = to_add.name
+
         if name in self._params:
             raise ValueAlreadyExistsError(Parameter, name)
-        if storage is not None:
-            param.set_storage(storage)
-        self._params[name] = param
-        return param
+
+        if multi:
+            if lower_limit:
+                to_add.set_to_multi(lower_limit)
+            else:
+                to_add.set_to_multi_at_least_one()
+
+        self._params[name] = to_add
+        return to_add
 
     def set_possible_param_order(self, line: str) -> None:
         params = line.split(' ') if len(line) else []
@@ -892,6 +909,27 @@ class HiddenNodeManagerMixin:
         return name in self._hidden_nodes
 
 
+class AlternativeNamesMixin(INamable):
+
+    def __init__(self, alternative_names: Iterable[str] = None, **kwargs):
+        super().__init__(**kwargs)
+        self._alternative_names = set(alternative_names or [])
+
+    def add_alternative_names(self, *alternative_names: str):
+        self._alternative_names |= set(alternative_names)
+
+    def has_name(self, name: str):
+        return super().has_name(name) or name in self._alternative_names
+
+    def has_name_in(self, names: Iterable[str]):
+        return any(map(self.has_name, names))
+
+    def get_all_names(self) -> list[str]:
+        return [self._name] + list(self._alternative_names)
+
+    def __hash__(self):
+        return hash((self.name, *self._alternative_names))
+
 #############
 # CLI parts #
 #############
@@ -1009,22 +1047,30 @@ class Node(ParameterManagerMixin, IResetable, ActionOnActivationMixin, FlagManag
         nodes = (self.add_node(to_add, action) for to_add, action in zip_longest(to_adds, actions))
         return tuple(nodes)
 
-    def add_node(self, to_add: str | VisibleNode, action: Callable = None) -> VisibleNode:
+    def add_node(self, to_add: str | VisibleNode, *alternative_names: Iterable[str], action: Callable = None) -> VisibleNode:
         if self._only_hidden:
             raise IncorrectStateError("Tried to add a visible node when only hidden option had been set")
         name, node = get_name_and_object_for_namable(to_add, VisibleNode)
         if name in self._visible_nodes:
             raise ValueAlreadyExistsError(VisibleNode, name)
+        node.add_alternative_names(*alternative_names)
         node.add_action(action)
         self._visible_nodes[name] = node
         return node
 
     def has_visible_node(self, node: str | VisibleNode) -> bool:
         name = get_name(node)
-        return name in self._visible_nodes
+        return any(node_instance.has_name(name) for node_instance in self._visible_nodes.values())
 
     def get_visible_node(self, name: str):
-        return self._visible_nodes[name]
+        try:
+            return self._visible_nodes[name]
+        except KeyError:
+            pass  # Fallback
+        try:
+            return next((node for node in self._visible_nodes.values() if node.has_name(name)))
+        except StopIteration:
+            raise KeyError
 
     def get_visible_nodes(self, *names: str) -> list[VisibleNode]:
         if not names:
@@ -1146,10 +1192,10 @@ class Node(ParameterManagerMixin, IResetable, ActionOnActivationMixin, FlagManag
         return next(iter(self._action_results), None)
 
 
-class VisibleNode(Node, ActionOnImplicitActivation):
+class VisibleNode(Node, ActionOnImplicitActivation, AlternativeNamesMixin):
 
-    def __init__(self, name: str, parameters: Iterable[str | Parameter] = None, param_storages: tuple[CliCollection] = (), **kwargs):
-        super().__init__(name=name, parameters=parameters, param_storages=param_storages, activated=False, **kwargs)
+    def __init__(self, name: str, *alternative_names: str, parameters: Iterable[str | Parameter] = None, param_storages: tuple[CliCollection] = (), **kwargs):
+        super().__init__(name=name, alternative_names=alternative_names, parameters=parameters, param_storages=param_storages, activated=False, **kwargs)
 
 
 class HiddenNode(Node, ActionOnCondition):  # TODO: refactor to remove duplications (active and inactive conditions should be a separate class
@@ -1207,7 +1253,10 @@ class CliCollection(DefaultStorage, SmartList, INamable, IResetable):
         return to_return
 
     def get_as_list(self) -> list[...]:
-        result = self.get_plain()
+        try:
+            result = self.get_plain()
+        except StopIteration:
+            result = []  # TODO: add test for this
         if isinstance(result, str):
             result = [result]
         if not isinstance(result, Iterable):
@@ -1453,15 +1502,14 @@ class Parameter(FinalNode, ActionOnCondition):
         flag.when_active_turn_off(self)
 
 
-class Flag(FinalNode, ActionOnImplicitActivation):
+class Flag(FinalNode, ActionOnImplicitActivation, AlternativeNamesMixin):
 
     def __init__(self, name, *alternative_names: str, storage: CliCollection = None, storage_limit: int = -1, storage_lower_limit=-1, default: default_type = None, flag_limit=-1, flag_lower_limit=-1):
         if flag_limit == -1:
             limit = storage.get_limit() if storage is not None else storage_limit
             flag_limit = limit if limit != -1 else 0
 
-        super().__init__(name, storage=storage, storage_limit=storage_limit, storage_lower_limit=storage_lower_limit, default=default, local_limit=flag_limit, local_lower_limit=flag_lower_limit, activated=False)
-        self._alternative_names = set(alternative_names)
+        super().__init__(name, alternative_names=alternative_names, storage=storage, storage_limit=storage_limit, storage_lower_limit=storage_lower_limit, default=default, local_limit=flag_limit, local_lower_limit=flag_lower_limit, activated=False)
         self._on_activation: SmartList[Callable] = SmartList()
 
     # Help
@@ -1474,29 +1522,11 @@ class Flag(FinalNode, ActionOnImplicitActivation):
             return super().get_long_description()
         raise NotImplementedError
 
-
     # Reset
 
     def reset(self):
         self.deactivate()
 
-    # Common
-
-    def add_alternative_names(self, *alternative_names: str):
-        self._alternative_names |= set(alternative_names)
-
-    def has_name(self, name: str):
-        return super().has_name(name) or name in self._alternative_names
-
-    def has_name_in(self, names: Iterable[str]):
-        return any(map(self.has_name, names))
-
-    def get_all_names(self) -> list[str]:
-        return [self._name] + list(self._alternative_names)
-
-    def __hash__(self):
-        return hash((self.name, *self._alternative_names))
-
 
 default_type = str | int | list[str | int] | None
 i_help_type = Node | Flag | Parameter | HiddenNode | VisibleNode
@@ -1515,5 +1545,8 @@ def get_name(arg: str | INamable) -> str:
 
 
 def storable_has_value(storable: IDefaultStorable, value: Any):
-    result = storable.get()
-    return result is not None and (result == value or value in result)
\ No newline at end of file
+    try:
+        result = storable.get()
+        return result == value or value in result
+    except StopIteration:
+        return False
\ No newline at end of file
